@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Number_types/include/CGAL/MP_Float.h $
+// $Id: MP_Float.h 3c08d12 %aI Marc Glisse
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Sylvain Pion
@@ -21,6 +22,7 @@
 #ifndef CGAL_MP_FLOAT_H
 #define CGAL_MP_FLOAT_H
 
+#include <CGAL/disable_warnings.h>
 #include <CGAL/number_type_basic.h>
 #include <CGAL/Algebraic_structure_traits.h>
 #include <CGAL/Real_embeddable_traits.h>
@@ -141,7 +143,7 @@ private:
     iterator i = v.begin();
     for (++i; *i == 0; ++i)
       ;
-    exp += i-v.begin();
+    exp += static_cast<exponent_type>(i-v.begin());
     v.erase(v.begin(), i);
   }
 
@@ -164,8 +166,8 @@ public:
    
     //Note: For Integer type, if the destination type is signed, the value is unchanged 
     //if it can be represented in the destination type)
-    low=static_cast<limb>(l & mask); //extract low bits from l 
-    high = (l - low) >> sizeof_limb; //extract high bits from l
+    low = static_cast<limb>(l & mask); //extract low bits from l
+    high= static_cast<limb>((l - low) >> sizeof_limb); //extract high bits from l
     
     CGAL_postcondition ( l == low + ( static_cast<limb2>(high) << sizeof_limb ) );
   }
@@ -232,7 +234,7 @@ public:
 
   exponent_type max_exp() const
   {
-    return v.size() + exp;
+    return exponent_type(v.size()) + exp;
   }
 
   exponent_type min_exp() const
@@ -305,7 +307,7 @@ public:
   // a value with an exponent close to 0.
   exponent_type find_scale() const
   {
-    return exp + v.size();
+    return exp + exponent_type(v.size());
   }
 
   // Rescale the value by some factor (in limbs).  (substract the exponent)
@@ -320,7 +322,7 @@ public:
   lsb(limb l)
   {
     unsigned short nb = 0;
-    for (; (l&1)==0; ++nb, l>>=1)
+    for (; (l&1)==0; ++nb, l=(limb)(l>>1) )
       ;
     return nb;
   }
@@ -350,7 +352,7 @@ public:
     CGAL_assertion(r.v.begin() != r.v.end());
     unsigned short nb = lsb(r.v[0]);
     r.v.clear();
-    r.v.push_back(1<<nb);
+    r.v.push_back((limb)(1<<nb));
     return (sign() > 0) ? r : -r;
   }
 
@@ -416,7 +418,7 @@ template <> class Algebraic_structure_traits< MP_Float >
     typedef Tag_true            Is_numerical_sensitive;
 
     struct Unit_part
-      : public std::unary_function< Type , Type >
+      : public CGAL::cpp98::unary_function< Type , Type >
     {
       Type operator()(const Type &x) const {
         return x.unit_part();
@@ -424,7 +426,7 @@ template <> class Algebraic_structure_traits< MP_Float >
     };
 
     struct Integral_division
-        : public std::binary_function< Type,
+        : public CGAL::cpp98::binary_function< Type,
                                  Type,
                                  Type > {
     public:
@@ -440,7 +442,7 @@ template <> class Algebraic_structure_traits< MP_Float >
 
 
     class Square
-      : public std::unary_function< Type, Type > {
+      : public CGAL::cpp98::unary_function< Type, Type > {
       public:
         Type operator()( const Type& x ) const {
           return INTERN_MP_FLOAT::square(x);
@@ -448,7 +450,7 @@ template <> class Algebraic_structure_traits< MP_Float >
     };
 
     class Gcd
-      : public std::binary_function< Type, Type,
+      : public CGAL::cpp98::binary_function< Type, Type,
                                 Type > {
       public:
         Type operator()( const Type& x,
@@ -458,7 +460,7 @@ template <> class Algebraic_structure_traits< MP_Float >
     };
 
     class Div
-      : public std::binary_function< Type, Type,
+      : public CGAL::cpp98::binary_function< Type, Type,
                                 Type > {
       public:
         Type operator()( const Type& x,
@@ -471,7 +473,7 @@ template <> class Algebraic_structure_traits< MP_Float >
 // Default implementation of Divides functor for unique factorization domains
   // x divides y if gcd(y,x) equals x up to inverses 
   class Divides 
-    : public std::binary_function<Type,Type,bool>{ 
+    : public CGAL::cpp98::binary_function<Type,Type,bool>{
   public:
     bool operator()( const Type& x,  const Type& y) const {  
       return internal::division(y,x).second == 0 ;
@@ -493,7 +495,7 @@ template <> class Real_embeddable_traits< MP_Float >
   public:
 
     class Sgn
-      : public std::unary_function< Type, ::CGAL::Sign > {
+      : public CGAL::cpp98::unary_function< Type, ::CGAL::Sign > {
       public:
         ::CGAL::Sign operator()( const Type& x ) const {
           return x.sign();
@@ -501,7 +503,7 @@ template <> class Real_embeddable_traits< MP_Float >
     };
 
     class Compare
-      : public std::binary_function< Type, Type,
+      : public CGAL::cpp98::binary_function< Type, Type,
                                 Comparison_result > {
       public:
         Comparison_result operator()( const Type& x,
@@ -511,7 +513,7 @@ template <> class Real_embeddable_traits< MP_Float >
     };
 
     class To_double
-      : public std::unary_function< Type, double > {
+      : public CGAL::cpp98::unary_function< Type, double > {
       public:
         double operator()( const Type& x ) const {
           return INTERN_MP_FLOAT::to_double( x );
@@ -519,7 +521,7 @@ template <> class Real_embeddable_traits< MP_Float >
     };
 
     class To_interval
-      : public std::unary_function< Type, std::pair< double, double > > {
+      : public CGAL::cpp98::unary_function< Type, std::pair< double, double > > {
       public:
         std::pair<double, double> operator()( const Type& x ) const {
           return INTERN_MP_FLOAT::to_interval( x );
@@ -851,14 +853,14 @@ class Real_embeddable_traits< Quotient<MP_Float> >
     : public INTERN_QUOTIENT::Real_embeddable_traits_quotient_base<
 Quotient<MP_Float> >{
 public:
-    struct To_double: public std::unary_function<Quotient<MP_Float>, double>{
+    struct To_double: public CGAL::cpp98::unary_function<Quotient<MP_Float>, double>{
          inline
          double operator()(const Quotient<MP_Float>& q) const {
             return INTERN_MP_FLOAT::to_double(q);
         }
     };
     struct To_interval
-        : public std::unary_function<Quotient<MP_Float>, std::pair<double,double> > {
+        : public CGAL::cpp98::unary_function<Quotient<MP_Float>, std::pair<double,double> > {
         inline
         std::pair<double,double> operator()(const Quotient<MP_Float>& q) const {
             return INTERN_MP_FLOAT::to_interval(q);
@@ -877,6 +879,8 @@ inline MP_Float max BOOST_PREVENT_MACRO_SUBSTITUTION(const MP_Float& x,const MP_
 // Coercion_traits
 CGAL_DEFINE_COERCION_TRAITS_FOR_SELF(MP_Float)
 CGAL_DEFINE_COERCION_TRAITS_FROM_TO(int, MP_Float)
+CGAL_DEFINE_COERCION_TRAITS_FROM_TO(float, MP_Float)
+CGAL_DEFINE_COERCION_TRAITS_FROM_TO(double, MP_Float)
 
 
 } //namespace CGAL
@@ -888,6 +892,7 @@ namespace Eigen {
     typedef CGAL::MP_Float Real;
     typedef CGAL::Quotient<CGAL::MP_Float> NonInteger;
     typedef CGAL::MP_Float Nested;
+    typedef CGAL::MP_Float Literal;
 
     static inline Real epsilon() { return 0; }
     static inline Real dummy_precision() { return 0; }
@@ -909,4 +914,5 @@ namespace Eigen {
 //specialization for Get_arithmetic_kernel
 #include <CGAL/MP_Float_arithmetic_kernel.h>
 
+#include <CGAL/enable_warnings.h>
 #endif // CGAL_MP_FLOAT_H
