@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Mesh_2/include/CGAL/Mesh_2/Face_badness.h $
+// $Id: Face_badness.h ee57fc2 %aI Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Laurent RINEAU
@@ -21,6 +22,9 @@
 #ifndef CGAL_MESH_2_FACE_BADNESS_H
 #define CGAL_MESH_2_FACE_BADNESS_H
 
+#include <CGAL/license/Mesh_2.h>
+
+
 namespace CGAL
 {
   namespace Mesh_2
