@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Hole_filling/do_not_use_DT3.h $
+// $Id: do_not_use_DT3.h ee57fc2 %aI Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Jane Tournois
@@ -21,6 +22,9 @@
 #ifndef CGAL_POLYGON_MESH_PROCESSING_INTERNAL_HOLE_FILLING_DO_NOT_USE_DT3
 #define CGAL_POLYGON_MESH_PROCESSING_INTERNAL_HOLE_FILLING_DO_NOT_USE_DT3
 
+#include <CGAL/license/Polygon_mesh_processing/meshing_hole_filling.h>
+
+
 #define CGAL_HOLE_FILLING_DO_NOT_USE_DT3
 
 #endif // CGAL_POLYGON_MESH_PROCESSING_INTERNAL_HOLE_FILLING_DO_NOT_USE_DT3
