@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/QP_solver/include/CGAL/QP_models.h $
+// $Id: QP_models.h 26c857a %aI Mael Rouxel-Labbé
 // SPDX-License-Identifier: GPL-3.0+
 // 
 //
@@ -396,7 +396,7 @@ namespace QP_model_detail {
   // maps a container to its begin-iterator, as specified by HowToBegin
   template<typename Container, typename Iterator, typename HowToBegin>
   struct Begin
-    : public CGAL::unary_function< Container, Iterator >
+    : public CGAL::cpp98::unary_function< Container, Iterator >
   {
     typedef Iterator result_type;
     result_type operator () ( const Container& v) const 
