@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Principal_component_analysis_LGPL/include/CGAL/centroid.h $
+// $Id: centroid.h 0698f79 %aI Sébastien Loriot
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Sylvain Pion
@@ -405,7 +406,7 @@ template < typename InputIterator,
 typename K::Point_3
 centroid(InputIterator begin, 
          InputIterator end, 
-         const K&,
+         const K& k,
          const typename K::Point_3*,
          CGAL::Dimension_tag<0>)
 {
@@ -418,7 +419,7 @@ centroid(InputIterator begin,
   unsigned int nb_pts = 0;
   while (begin != end) 
   {
-    v = v + (*begin++ - ORIGIN);
+    v = v + k.construct_vector_3_object()(ORIGIN, *begin++);
     nb_pts++;
   }
   return ORIGIN + v / (FT)nb_pts;
