@@ -12,8 +12,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Convex_hull_3/include/CGAL/convex_hull_3.h $
+// $Id: convex_hull_3.h 885cc02 %aI Sébastien Loriot
 // SPDX-License-Identifier: GPL-3.0+
 //
 //
@@ -182,13 +182,13 @@ struct Is_cartesian_kernel< Convex_hull_traits_3<Kernel, PolygonMesh, Tag_true>
 {
   // Rational here is that Tag_true can only be passed by us since it is not documented
   // so we can assume that Kernel is a CGAL Kernel
-  typedef boost::is_same<typename Kernel::Kernal_tag, Cartesian_tag> type;
+  typedef typename boost::is_same<typename Kernel::Kernel_tag, Cartesian_tag>::type type;
 };
 
 // Predicate internally used as a wrapper around has_on_positive_side
 // We provide a partial specialization restricted to the case of CGAL Cartesian Kernels with inexact constructions below
 //template <class Traits,class Tag_use_advanced_filtering=typename Use_advanced_filtering<Traits>::type >
-template <class Traits, class Is_CK = Is_cartesian_kernel<Traits> >
+template <class Traits, class Is_CK = typename Is_cartesian_kernel<Traits>::type >
 class Is_on_positive_side_of_plane_3{
   typedef typename Traits::Point_3 Point_3;
   typename Traits::Plane_3 plane;
